--[[
************************************************************************
Project				: SKG
Author				: zhinjio
Project Revision	: r130
Project Date		: 20090126004334

File				: Comms.lua
Commit Author		: zhinjio
Commit Revision		: 128
Commit Date			: 20090126002630
************************************************************************
Description	:
	This file handles all the listeners for incoming communications, and
	routes the commands to the appopriate functions.

	It should be noted that the addon does not do really *anything* just
	native to itself. It almost always calls a COMM so that it essentially
	will always be a client to its own COMM calls.

TODO		:

************************************************************************
(see bottom of file for changelog)
************************************************************************
--]]
local MODNAME = "SKG"
local FULLNAME = MODNAME
local SKG = LibStub( "AceAddon-3.0" ):GetAddon( MODNAME, "AceEvent-3.0" )
local L = LibStub:GetLibrary( "AceLocale-3.0" ):GetLocale( MODNAME )

local addon = SKG

-- My messages are all going to be complex types, and very rarely just strings. As
-- such, I need to serialize all my sends and deserialize all receives. In addition,
-- my receiving function is going to always treat the first argument as a "token",
-- indicating the type of communication, and what function to send the payload to

addon.OCR = {}

function addon.SendComm( token, payload, distribution, target )
	local newPayload = SKG:Serialize( token, payload )
	addon:SendCommMessage( SKG_CHANNEL_PREFIX, newPayload, distribution, target, "BULK" )
end

function addon:OnCommReceived( prefix, message, distribution, sender )
	addon.DoIt( sender, SKG:Deserialize( message ) )
end

function addon.DoIt( sender, result, ... )
	if ( result ) then
		local token, payload = ...
		addon.Debug( "OCR : " .. token )
		addon.OCR[token]( sender, payload )
	else
		local errormsg = ...
		addon.Debug( "Unable to Deserialize message : " .. errormsg ) 
	end
end

addon.OCR.DEBUG = function( sender, message )
	addon.Debug( "Comm: Debug (" .. sender .. ") : " .. message )
end

--[[
************************************************************************
CHANGELOG:

Date : 1/25/09
	Rewrite begins
Date : 12/21/08
	Added Death Knight compatibility
Date : 10/26/08
	Moving RegisterComm to OnInit
Date : 10/25/08
	Initial attempt with Ace3 instead of LibRock. Added header/footer.
	Added functions to serialize/deserialize, and converted functions
	to a table SKG_OCR (OnCommReceive) which then has sub-functions for
	each token. Thats about as close to the old method as I can get.
************************************************************************
]]--